/* pentbug.c             Thomas R. Nicely          2011.08.20.0700
 *
 * Freeware copyright (c) 2011 Thomas R. Nicely <http://www.trnicely.net>.
 * Released into the public domain by the author, who disclaims any legal
 * liability arising from its use.
 *
 * DESCRIPTION:
 *
 * Tests for a numerical calculation (FDIV) bug present in the FPU of
 * many older Pentium CPUs (60 MHz, 66 MHz, and a few 90 MHz versions
 * manufactured prior to November, 1994). The flaw was first detected
 * internally by Intel in May, 1994, but was not publicly known until
 * announced by Thomas R. Nicely on 30 October 1994.
 *
 * Certain quotients, as computed by the fdiv family of x87 instructions,
 * were returned incorrectly in scattered cases.  The fprem family of
 * instructions was also affected. The errors are due to missing entries
 * in a hardware lookup table used in the FPU's division algorithm (the
 * Sweeney-Robertson-Tocher radix-4 algorithm, generating two quotient
 * bits per clock).
 *
 * Note that the denominators actually producing erroneous returns are
 * rare exceptions. The random sampling incidence appears to be about
 * 1 in 8.77e9 for two-operand divisions and 1 in 1.26e9 for pure
 * reciprocals. IBM claimed in December, 1994, that errors were far
 * more frequent than this (implying at one point that 4000 such errors
 * could be expected to occur each day), but there is little or no
 * real-world evidence to back up their claim. Indeed, no user ever
 * detected or reported any such error prior to Nicely's discovery,
 * and no user has reported any uncontrived instance of such an error
 * in the ensuing sixteen years.
 *
 * The worst-case error known is produced by the quotient
 *
 *			4195835.0/3145727.0
 *
 * as posted by Tim Coe of V*tesse Semiconductor on 1994.11.14.  This
 * produces only 12 correct bits (14 significant bits) in the quotient,
 * and a relative error of 1 part in 16,390. Other similar quotients
 * also exist.
 *
 * The flaw can be made more glaring by calculating the quantity
 *
 *     4195835.0 - 3145727.0*(4195835.0/3145727.0)
 *
 * which equals zero; the flawed Pentium CPU will produce a result
 * of 256 instead. This is the computation (also suggested by Coe)
 * used in this test code.
 *
 * Various workarounds exist for the bug. These include disabling
 * the FPU, which slows down floating-point calculations by one or
 * two orders of magnitude; calling a floating-point emulator library
 * rather than using FPU hardware instructions; or trapping floating
 * point calls and inserting a check on divisions.
 *
 * For additional discussion, see
 * <http://www.trnicely.net/pentbug/pentbug.html> and related documents.
 *
 * The compiler for this code must generate FPU hardware instructions
 * for the arithmetic operations, rather than calling a floating-point
 * emulator library; this is the default for most C compilers. Also,
 * optimization should be disabled.
 *
 * The accompanying executable PENTBUG.EXE is a 32-bit command-line
 * executable of this code. The accompanying executable BUG16BIT.EXE
 * is a 16-bit command-line executable of this code; it is intended
 * for use with standalone DOS and systems prior to Windows 95, which
 * might well be encountered in conjunction with flawed chips. The
 * 16-bit executable may not run on 64-bit versions of Windows.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

static const char signature[]=
  "\n __pentbug.c__Version 2011.08.20.0700__Freeware copyright (c) 2011"
  "\n Thomas R. Nicely <http://www.trnicely.net>. Released into the"
  "\n public domain by the author, who disclaims any legal liability"
  "\n arising from its use.\n";

/**********************************************************************/
int main(void)
{
double lfNum1=4195835.0, lfNum2=4195835.0, lfDenom1=3145727.0,
  lfDenom2=3145727.0, lfQuot, lfProd, lfZero;

/* The duplicate variables are used in an effort to foil compiler
   optimization and compile-time evaluation of numeric literals. */

lfQuot=lfNum1/lfDenom1;
lfProd=lfDenom2*lfQuot;
lfZero=lfNum2 - lfProd;  /* At least it _should_ be zero! */
printf("\n This system produces the");
if(fabs(lfZero) < 0.5e-8)  /* Ignore benign rounding errors */
  {
  printf(" correct result:\n");
  printf("\n 4195835.0 - 3145727.0*(4195835.0/3145727.0) = %.8f\n",
    lfZero);
  printf("\n and therefore is free from the Pentium FDIV flaw,");
  printf("\n which produces an erroneous value of 256.\n");
  }
else
  {
  printf(" ERRONEOUS result:\n");
  printf("\n 4195835.0 - 3145727.0*(4195835.0/3145727.0) = %.8f",
    lfZero);
  printf("  (should be zero)\n");
  printf("\n and therefore contains a FLAWED PENTIUM PROCESSOR.\n");
  }
return(0);
}
/**********************************************************************/
